package dev.fyloz.colorrecipesexplorer.logic

import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import dev.fyloz.colorrecipesexplorer.config.properties.CreSecurityProperties
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDto
import dev.fyloz.colorrecipesexplorer.logic.account.DefaultJwtLogic
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import io.mockk.clearAllMocks
import io.mockk.spyk
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import java.util.*
import kotlin.test.assertEquals
import kotlin.test.assertFalse

class DefaultJwtLogicTest {
    private val objectMapper = jacksonObjectMapper()
    private val securityProperties = CreSecurityProperties().apply {
        jwtSecret = "exBwMbD9Jw7YF7HYpwXQjcsPf4SrRSSF5YTvgbj0"
        jwtDuration = 1000000L
    }

    private val jwtLogic = spyk(DefaultJwtLogic(objectMapper, securityProperties))

    private val permissions = listOf(Permission.VIEW_RECIPES, Permission.READ_FILE, Permission.VIEW_CATALOG)
    private val user = UserDto(999L, "Unit test", "User", "", null, permissions)
    private val userDetails = UserDetails(user)

    private val userJwt = "eyJhbGciOiJIUzM4NCJ9.eyJzdWIiOiI5OTkiLCJleHAiOjE2Njk2MTc1MDcsInBlcm1zIjoiWzIsMCwzXSIsInR5cGUiOjB9.bg8hbTRsWOcx4te3L0vi8WNPXWLZO-heS7bNsO_FBpkRPy4l-MtdLOa6hx_-pXbZ"
    private val groupTokenJwt = "eyJhbGciOiJIUzM4NCJ9.eyJzdWIiOiJhMDIyZWU3YS03NGY5LTNjYTYtYmYwZC04ZTg3OWE2NjRhOWUifQ.VaRqPJ30h8WUACPf8wVrjaxINQcc9xnbzGOcMesW_PbeN9rEGzgkgFEuV4TRGlOr"
    private val groupTokenId = UUID.nameUUIDFromBytes("Unit test token".toByteArray())

    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun buildUserJwt_normalBehavior_buildJwtWithValidSubject() {
        // Arrange
        // Act
        val jwt = jwtLogic.buildUserJwt(userDetails)

        // Assert
        val parsedJwt = jwtLogic.parseUserJwt(jwt)
        assertEquals(user.id.toString(), parsedJwt.id)
    }

    @Test
    fun buildUserJwt_normalBehavior_buildJwtWithValidType() {
        // Arrange
        // Act
        val jwt = jwtLogic.buildUserJwt(userDetails)

        // Assert
        val parsedJwt = jwtLogic.parseUserJwt(jwt)
        assertFalse(parsedJwt.isGroup)
    }

    @Test
    fun buildUserJwt_normalBehavior_buildJwtWithValidPermissions() {
        // Arrange
        // Act
        val jwt = jwtLogic.buildUserJwt(userDetails)

        // Assert
        val parsedJwt = jwtLogic.parseUserJwt(jwt)
        assertEquals(userDetails.authorities, parsedJwt.authorities)
    }

    @Test
    fun buildGroupTokenIdJwt_normalBehavior_buildJwtWithValidSubject(){
        // Arrange
        // Act
        val jwt = jwtLogic.buildGroupTokenIdJwt(groupTokenId)

        // Assert
        val parsedGroupId = jwtLogic.parseGroupTokenIdJwt(jwt)
        assertEquals(groupTokenId, parsedGroupId)
    }

    @Test
    fun parseUserJwt_normalBehavior_returnsUserWithValidId() {
        // Arrange
        // Act
        val user = jwtLogic.parseUserJwt(userJwt)

        // Assert
        assertEquals(userDetails.id, user.id)
    }

    @Test
    fun parseUserJwt_normalBehavior_returnsUserWithValidType() {
        // Arrange
        // Act
        val user = jwtLogic.parseUserJwt(userJwt)

        // Assert
        assertFalse(user.isGroup)
    }

    @Test
    fun parseUserJwt_normalBehavior_returnsUserWithValidPermissions() {
        // Arrange
        // Act
        val user = jwtLogic.parseUserJwt(userJwt)

        // Assert
        assertEquals(userDetails.authorities, user.authorities)
    }

    @Test
    fun parseGroupTokenId_normalBehavior_returnsValidGroupTokenId() {
        // Arrange
        // Act
        val parsedGroupTokenId = jwtLogic.parseGroupTokenIdJwt(groupTokenJwt)

        // Assert
        assertEquals(groupTokenId, parsedGroupTokenId)
    }
}
